from flask import Blueprint

# bp = Blueprint('webui', __name__, template_folder='templates', static_folder='static', static_url_path='/static') 


# from .routes import WebUIView

# from .mangosteen_base import MangosteenBlueprint
# from .routes import WebUIView

zbp = Blueprint( 
                    name='webui',
                    import_name=__name__,  
                    url_prefix='/', 
                    template_folder='templates', 
                    static_folder='static', 
                    static_url_path='/' 
                )

    # def __init__( self, blueprint_name, blueprint_module, route_class, url_prefix='/', template_folder='templates', static_folder='static', static_url_path='/static' ):
    #     self._bp = Blueprint(   name=blueprint_name, 
    #                             import_name=blueprint_module, 
    #                             static_folder=static_folder,
    #                             static_url_path=static_url_path,
    #                             template_folder=template_folder,
    #                             url_prefix=url_prefix  )

    #     self._route_class = route_class

    # def get_blueprint(self):
    #     return self._bp

    # def get_view(self):
    #     return self._route_class 
                


import  json 
from jsonschema import validate as json_schema_validate
from jsonschema.exceptions import ValidationError


from .web_field import WebField
from .mangosteen_base import MangosteenBase 
from dataobjtools.selector import Selector
import importlib

from dataclasses import dataclass, field
from typing import List, Callable

@dataclass
class DataUI_Error():
	no: str
	message: str
	validation: dict= field(default_factory=dict)
	data: dict= field(default_factory=dict)

@dataclass
class DataUI_Response():
	success: bool
	dbrec_list: list
	request_id: str = ""
	schema: dict = field(default_factory=dict)
	http_status: int = 000
	error: DataUI_Error = None # DataUI_Error(no=None,message=None, validation={},data={} )

	def to_http(self):
		ret_data = {
					'request_id': self.request_id,
					'success': self.success,
					'data': self.data,
					'schema': self.schema
		}
		if self.error.no : ret_data['error'] = self.error.__dict__
		return json.dumps( ret_data ), self.http_status

	# def to_dict(self):


@dataclass
class DataUI_Response_List():
	success: bool
	response_list: List[DataUI_Response] = field(default_factory=list)
	http_status: int = 000
	error: DataUI_Error = None #DataUI_Error(no=None,message=None, validation={},data={} )

	def to_http(self):
		ret_data = { 'success': self.success, 'data_list':[]}

		for data_item in self.response_list:
			ret_data['data_list'].append( data_item.__dict__  )
		
		if self.error : ret_data['error'] = self.error.__dict__
		
		# breakpoint()
		return json.dumps( ret_data ), self.http_status
		


class DataUIModel(MangosteenBase):
	#####################################################################################
	def __init__(self:dict,   form_field_validation_schema:dict, session, logger ):

		super().__init__(app = None, logger = logger )

		DataUIModel.validate_schema( form_field_validation_schema )

		# self.update_obj_list = update_obj
		self.data_schema = form_field_validation_schema 
		
		self.web_schema = WebField( form_field_validation_schema, logger=logger )
		# self.logger = logger
		self.session = session

		# pass
	@staticmethod
	def validate_schema( json_input_schema):
		validation_schema = {
							"type": "array",
							"items": 
							{ 	"type" : "object",
								"properties" : 
								{ "module_name":{"type":"string" },
									"table_obj":{"type":"string" },
									"fields": 
									{ "type": "object",
										"patternProperties": 
										{	".*": 
											{ 	"type": "object",
												"properties": 
												{ 	"field_db":{"type":"string" },
													"key":{"type":"boolean" },
													"validation":{
																	"type": "object",
																	"properties": {
																					"required":{"type":"boolean"},
																					"text_min_len":{"type":"integer"},
																					"text_max_len":{"type":"integer"}
								} 	}	}	}	}	}	}	}
							}
		try:
			json_schema_validate( instance= json_input_schema , schema=validation_schema   )
		except  Exception as e:
			print(e)
			return False

		return True
	
	#####################################################################################
	def validate_schema_for_bulk_update(self, data ): 
		return True

	#####################################################################################
	def get_webfield_value( self, web_data, field_html_name):
		for field in web_data:	
			if field[ 'id'] == field_html_name: return field['value']
		
		return None

	#####################################################################################
	def get_field_schema( self, obj_name:str, field_html_name:str):
		
		for table_obj_rec in self.data_schema:
			if obj_name == table_obj_rec['table_obj']:
				return table_obj_rec['fields'].get(field_html_name)


		# table_obj_rec = self.data_schema.get( obj_name )

		# if table_obj_rec:
			
		return None

	#####################################################################################
	def data_validate( self, submit_data ): 
		validation_result = {}
		self.log_debug("validating data")
		self.log_debug( submit_data )

		validation_result = self.web_schema.validate(  submit_data )
		if validation_result['success']: self.log_debug( 'Validation checks completed - result: passed' )
		else: self.log_error('Validation checks completed - result: failed')
		self.log_debug("****:" + json.dumps(   validation_result ) )
		
		return validation_result


	#####################################################################################
	def data_get_ajax( self, search_dict ):  
		
		data_obj_list = self.data_get( search_dict )
		
		if data_obj_list: return DataUI_Response(success=True, data_list=data_obj_list, schema=self.data_schema, http_status=200).to_http()
		
		return DataUI_Response(success=False, error=DataUI_Error(no='101', message='data not found'), http_status=500) 

	#####################################################################################
	def data_get( self, search_dict, ret_db_obj=False):
		data_obj_list = []
		## Example of talbe fields in self.data_schema
		# 	"SiteMain":{ 	
		# 		# "table":"SiteMain",
		# 		# "fields":{
		# 					"si_site_id":{"field_db":"site_id",   "key":True},
		# 					"si_site_name":{"field_db":"site_name", "validation":{"required":True, "text_min_len":3, "text_max_len":20} },
		# 					"si_site_code":{"field_db":"site_code",   "validation":{"required":True,  "text_max_len":5} },
		# 		# } }
		#Loop through each of the json table schame descriptions
		for obj_name, data_table in self.data_schema.items():
			# if obj_name in self.update_obj_list:
			db_fields = self.data_get_table_fields( data_table['fields'] , search_dict )
			self.log_debug(f"{obj_name} Read table: {data_table['module_name']}::{ data_table['table_obj'] } with search keys {db_fields['keys'] }" )	
			
			if db_fields:
				data_class_ref = getattr(importlib.import_module( data_table['module_name'] ), data_table['table_obj'] )	#Get ref to table object name dynamically
				data_obj = self.session.query( data_class_ref ).filter_by( **db_fields['keys'] ).all()

				for data_item in data_obj:
					if ret_db_obj: data_obj_list.append( data_item  )
					else: data_obj_list.append( data_item.to_dict()  )

		[ self.log_debug( f'Data from query: {data_item}')  for data_item in data_obj_list ] 
		return data_obj_list
				
	#####################################################################################
	def data_delete_ajax( self, submit_data: dict )-> str: 
		return self._data_process_ajax( submit_data, "del").to_http()

	#####################################################################################
	def data_update_ajax( self, submit_data:dict ): 
		return self._data_process_ajax( submit_data, "add").to_http()
		
	#####################################################################################
	def data_bulk_update_ajax( self, submit_data:dict )->DataUI_Response_List: 
		if not submit_data:
			self.log_error('No data given')  
			return json.dumps({'success':False}), 500 
		else:
			if not self.validate_schema_for_bulk_update(submit_data): 
				return DataUI_Response_List(success=False, error=DataUI_Error(no='001', message='Invalid Schema', validation=validation_result), http_status=500).to_http()

			return_data = []
			success = True
			
			for transaction in submit_data:	#Go through each transaction
				transcation_result = self._data_process_ajax( transaction['data'], transaction['transaction'] )
				if transcation_result:
					return_data.append( transcation_result )
					success = success & transcation_result.success	#see if successful or not
				else: 
					# breakpoint()
					success = False
			# if success_rate: return json.dumps( return_data ), 200
			if success: return DataUI_Response_List(success=True, response_list=return_data, http_status=200).to_http()

			return DataUI_Response_List(success=False, error=DataUI_Error(no='002', message='Some transactions failed'), response_list=return_data, http_status=500).to_http()


	#####################################################################################
	def _data_process_ajax( self, submit_data_row:dict, action:str )-> DataUI_Response: 
		if not submit_data_row:
			self.log_error('No data given')  
			return DataUI_Response(success=False, error=DataUI_Error(no='003', message='No data given'), http_status=500) 
		else:
			validation_result = self.data_validate( submit_data_row)
			if validation_result['success']:
				if action == 'add':
					data_obj_list = self.data_process( submit_data_row, self._data_process_add )
				elif action == 'edit':
					data_obj_list = self.data_process( submit_data_row, self._data_process_edit )
				elif action == 'del':
					data_obj_list = self.data_process( submit_data_row, self._data_process_del )
				else:
					self.log_error('No data given')  
					return DataUI_Response(success=False, error=DataUI_Error(no='004',message=f"Cannot recognize action[{action}]"), http_status=500) 
				
				if data_obj_list:
					return DataUI_Response(success=True, dbrec_list=data_obj_list, schema=self.data_schema, http_status=200) 
			else:
				return DataUI_Response(success=False, error=DataUI_Error(no='005',message=f"Validation failed", validation=validation_result), http_status=500) 
				

	#####################################################################################
	# 	"SiteMain":{ 	
	# 		# "table":"SiteMain",
	# 		# "fields":{
	# 					"si_site_id":{"field_db":"site_id",   "key":True},
	# 					"si_site_name":{"field_db":"site_name", "validation":{"required":True, "text_min_len":3, "text_max_len":20} },
	# 					"si_site_code":{"field_db":"site_code",   "validation":{"required":True,  "text_max_len":5} },
	# 		# } }
	#Loop through each of the json table schame descriptions
	def data_process(self, web_data:dict, fn_data_process: Callable)->list:
		data_obj_list = [] 
		# breakpoint()
		for data_table in self.data_schema : 
			db_fields = self.data_get_table_fields( data_table['fields'] , web_data )
			self.log_debug(f"Modify table: {data_table['module_name']}::{ data_table['table_obj'] } with search keys {db_fields['keys'] }" )	
			
			data_class_ref = getattr(importlib.import_module( data_table['module_name'] ), data_table['table_obj'] )	#Get ref to table object name dynamically
			
			data_obj = fn_data_process( db_fields, data_class_ref )
			if data_obj: data_obj_list.append( data_obj.to_dict()  )
			
		if data_obj: self.session.commit()
		
		return data_obj_list 

	#####################################################################################
	def _data_process_add(self, db_fields:dict, data_class_ref:MangosteenBase ):
		#If not found, create new object
		data_obj = data_class_ref( **db_fields['fields'] )
		self.session.add(data_obj)  
		# self.session.flush()	#specifically flush to get the ID number
		return data_obj

	#####################################################################################
	def _data_process_edit(self, db_fields:dict, data_class_ref:MangosteenBase ):
		#Search to see if the record exists
		data_obj = self.session.query( data_class_ref ).filter_by( **db_fields['keys'] ).first()
		
		if data_obj:  #If found, then update fields
			for db_field_name in db_fields['fields']:
				setattr( data_obj, db_field_name, db_fields['fields'][ db_field_name ]  )
		return data_obj

	#####################################################################################
	def _data_process_del(self, db_fields:dict, data_class_ref:MangosteenBase ):
	
		if db_fields['keys']: 
			data_obj = self.session.query( data_class_ref ).filter_by( **db_fields['keys'] ).first()
			
			self.session.query( data_class_ref ).filter_by( **db_fields['keys'] ).delete()
			# self.session.commit()
			return data_obj
		return None


	#####################################################################################
	def data_add(self, web_data):
		data_obj_list = []
		# breakpoint()
		## Example of talbe fields in self.data_schema
		# 	"SiteMain":{ 	
		# 		# "table":"SiteMain",
		# 		# "fields":{
		# 					"si_site_id":{"field_db":"site_id",   "key":True},
		# 					"si_site_name":{"field_db":"site_name", "validation":{"required":True, "text_min_len":3, "text_max_len":20} },
		# 					"si_site_code":{"field_db":"site_code",   "validation":{"required":True,  "text_max_len":5} },
		# 		# } }
		#Loop through each of the json table schame descriptions
		for data_table in self.data_schema :
			# if obj_name in self.update_obj_list:
			db_fields = self.data_get_table_fields( data_table['fields'] , web_data )
			self.log_debug(f"Modify table: {data_table['module_name']}::{ data_table['table_obj'] } with search keys {db_fields['keys'] }" )	
			
			data_class_ref = getattr(importlib.import_module( data_table['module_name'] ), data_table['table_obj'] )	#Get ref to table object name dynamically
			data_obj = None

			#If not found, create new object
			data_obj = data_class_ref( **db_fields['fields'] )
			self.session.add(data_obj)  
			self.session.flush()	#specifically flush to get the ID number
			data_obj_list.append( data_obj.to_dict()  )

		self.session.commit()
		
		return data_obj_list 

	#####################################################################################
	def data_update(self, web_data):
		data_obj_list = []
		# breakpoint()
		## Example of talbe fields in self.data_schema
		# 	"SiteMain":{ 	
		# 		# "table":"SiteMain",
		# 		# "fields":{
		# 					"si_site_id":{"field_db":"site_id",   "key":True},
		# 					"si_site_name":{"field_db":"site_name", "validation":{"required":True, "text_min_len":3, "text_max_len":20} },
		# 					"si_site_code":{"field_db":"site_code",   "validation":{"required":True,  "text_max_len":5} },
		# 		# } }
		#Loop through each of the json table schame descriptions
		for data_table in self.data_schema :
			# if obj_name in self.update_obj_list:
			db_fields = self.data_get_table_fields( data_table['fields'] , web_data )
			self.log_debug(f"Modify table: {data_table['module_name']}::{ data_table['table_obj'] } with search keys {db_fields['keys'] }" )	
			
			data_class_ref = getattr(importlib.import_module( data_table['module_name'] ), data_table['table_obj'] )	#Get ref to table object name dynamically
			data_obj = None

			#Search to see if the record exists
			data_obj = self.session.query( data_class_ref ).filter_by( **db_fields['keys'] ).first()
			
			if data_obj:  #If found, then update fields
				for db_field_name in db_fields['fields']:
					setattr( data_obj, db_field_name, db_fields['fields'][ db_field_name ]  )
			data_obj_list.append( data_obj.toz_dict()  )

		self.session.commit()
		
		return data_obj_list 

	#####################################################################################
	def data_delete(self, web_data):
		for data_table in self.data_schema:
			db_fields = self.data_get_table_fields( data_table['fields'] , web_data )
			self.log_debug(f" Delete table: {data_table['module_name']}::{ data_table['table_obj'] } with search keys {db_fields['keys'] }" )

			data_class_ref = getattr(importlib.import_module( data_table['module_name'] ), data_table['table_obj']  )	#Get ref to table object name dynamically
			data_obj = None
			if db_fields['keys']: 
				data_obj = self.session.query( data_class_ref ).filter_by( **db_fields['keys'] ).delete()
				self.session.commit()
		return True


	#####################################################################################
	def data_get_table_fields(self, table_schema_fields, web_data ):
		data_obj_list = []
		db_fields = {'keys':{}, 'fields':{} } 
		# breakpoint()
		for field_name, field in table_schema_fields.items():	#Loop through each of the fields in a given table
			
			new_field = Selector().dict_search( web_data, {'id': field_name  } )
			if not new_field: 
				self.log_error(f'Could not find key [{field_name}] within web_data:[{web_data}]')
			else:
				# self.logger.debug(f"Adding field [{field_name}] value of: [{ new_field['value']  }]")
				if field.get("key") :
					if new_field['value']: db_fields[ "keys"][ field["field_db"] ] = new_field['value'] 
				else:
					db_fields[ "fields"][ field["field_db"] ] = new_field['value'] 

		self.log_debug(f"fields: {db_fields}")
		return db_fields 

 


import json , re

from flask import url_for, current_app
# from flask_wtf import csrf_token
# from zohavi.zcommon import Utils


##############################################################################################################


class JCFunc:

	CSRF_TOKEN_NAME = "_csrf_token"

	def get_csrf_token_name():
		return JCFunc.CSRF_TOKEN_NAME

	def set_csrf_token(myapp, class_name):
		# breakpoint()
		csrf_str  = f'<wc-hidden id = "{JCFunc.CSRF_TOKEN_NAME}" name="{JCFunc.CSRF_TOKEN_NAME}" '  
		csrf_str += f'value="{current_app.jinja_env.globals["csrf_token"]() }"'
		csrf_str += f'class="{class_name}" ></wc-hidden>'
		return 	csrf_str
				
			 	

	def load_template_funcions(jinja_env):
		for method in dir(JCFunc):
			if method.startswith('__') is False:
				jinja_env.globals[ 'jc_' + method] = getattr(JCFunc, method)


	######################################################################################################
	# Test function
	def test():
		return "hello JCFunc world"

	def defaulter(value, default):
		return value if value else default
	# def breakpoint():
	# 	breakpoint()


	def breakpoint(data):
		breakpoint()

	######################################################################################################
	# Helper function 
	def dict_to_json_str(inp_dict, field_list):
		ret_json = {}

		for field in field_list:
			if field in inp_dict: ret_json[field] = inp_dict[field]

		return json.dumps( ret_json )
		# pass

	######################################################################################################
	# Helper function 
	def list_dict_to_json_str(inp_list, field_list):
		ret_json = []

		for item in inp_list:
			item_dict = {}
			for field in field_list:
				if field in item: item_dict[field] = item[field]
			if item_dict: ret_json.append( item_dict )

		return json.dumps( ret_json )
		# pass
	
	######################################################################################################
	# Take an existing list or dict, and then convert the field names from A to B
	def translate_obj(obj, field_mapping, field_inserts ): 
		new_obj = None

		if not obj: return []	

		if isinstance(obj, list):
			new_obj = []
			for item in obj:	#Look through each of the rows
				curr_row = {}

				for field in item:
					if field in field_mapping:	#for each of the field mapping - "field" = original field
						curr_row[ field_mapping[field] ] = item[ field ]
					else:
						curr_row[ field ] = item[ field ]

				# if field_inserts: breakpoint() 
				for curr_insert in field_inserts:	#Go through each of the insert items
					search_match = True
					for search_item in curr_insert["search"]:	#check if the search criteria matches.  if so, then add it to the item
					 	if not ( search_item in item and item[search_item] == curr_insert["search"][ search_item ] ) :
					 		search_match = False

					if search_match:  curr_row.update (    curr_insert[ "insert" ]  )
							
				new_obj.append( curr_row )

		else:
			raise Exception( 'error')
		# print( json.dumps( new_obj ) )
		# print('deondone')
		return json.dumps( new_obj )
		# return json.dumps( obj )
		# breakpoint()
	######################################################################################################
	# Helper function for set_elt_attribs() to render each attribute in a HTML tag
	def concat_dirs(base_dir, *sub_dirs):
		new_dir = base_dir;
		for curr_sub_dir in sub_dirs :
			if curr_sub_dir:
				# logger.debug(f"adding {new_dir} + {curr_sub_dir} ")
				add_sub_dir = curr_sub_dir	
				if curr_sub_dir[0] == '/': curr_sub_dir = curr_sub_dir[1:]

				if new_dir[-1] == "/": 
					new_dir += curr_sub_dir
				else:  
					new_dir += "/" + curr_sub_dir
		if new_dir and new_dir[-1] != "/":
			new_dir += "/"

		# logger.debug("returning adding:" + new_dir)
		return new_dir


	######################################################################################################
	# Helper function for set_elt_attribs() to render each attribute in a HTML tag
	def _set_elt_attribs_print_str(item, data_value ): 
		if isinstance(data_value, dict):
			# print( f"####:{data_value}")
			return f" {item}='{ json.dumps(data_value) }' "
		return f' {item}="{data_value}" '

	

	######################################################################################################
	# Set the attributes of a HTML tag where **defaults_dict will be added if no entry in the field provided
	def set_elt_attribs(field, **defaults_dict ):
		attrib_str = "" 
		for item in defaults_dict:
			if item in field: 
				attrib_str += JCFunc._set_elt_attribs_print_str( item, field[item] )
			else:
				attrib_str += JCFunc._set_elt_attribs_print_str( item, defaults_dict[item] )   

		print( f"****:{attrib_str}")
		return attrib_str
	
	# {"class":"" , "message_type=opt[error,info,warn,success]":"", "message_header":"", "message":""}, ["id"]);   
	######################################################################################################
	def fill_message( message_data:dict, overrides:dict={} ):		
		attrib_str = ""

		attrib_str += f'id="{ 				JCFunc._fill_attribs_get_attrib_value( message_data, "id", overrides) }"  ' 
		attrib_str += f'class="{ 			JCFunc._fill_attribs_get_attrib_value( message_data, "class", overrides) }"  ' 
		attrib_str += f'message_type="{ 	JCFunc._fill_attribs_get_attrib_value( message_data, "message_type", overrides) }"  ' 
		attrib_str += f'message_header="{ 	JCFunc._fill_attribs_get_attrib_value( message_data, "message_header", overrides) }"  ' 
		attrib_str += f'message="{ 			JCFunc._fill_attribs_get_attrib_value( message_data, "message", overrides) }"  ' 
			
		return attrib_str
	
	######################################################################################################
	def fill_attribs( attrib_type: str, table_obj:str, field_id:str, data_ui: dict, data_value: any=None, overrides:dict={} ):
		attrib_filler_func = {
								'wc-input': JCFunc._fill_attribs_wc_input_text
		}
		# breakpoint()
		if attrib_type in attrib_filler_func: return attrib_filler_func[attrib_type](table_obj, field_id, data_ui, data_value, overrides)
		return f"ERROR: ATTRIB_TYPE [{attrib_type}] not found"

	######################################################################################################
	def _fill_attribs_wc_input_text(table_obj:str, field_id:str, data_ui: dict, data_value: any, overrides:dict) -> str:
		# breakpoint()
		field_schema = data_ui.get_field_schema( table_obj ,  field_id )
		if not field_schema:
			raise ValueError( f"Either table object:[{table_obj}] or field id[{field_id}] not found in the web data ui schema passed.  Check JSON schema and Jinja template entry for match")
		
		attrib_str =  f'id="{ field_id }"  ' 
		attrib_str += f'placeholder="{ JCFunc._fill_attribs_get_attrib_value( field_schema, "placeholder", overrides) }"  ' 
		attrib_str += f'label="{  JCFunc._fill_attribs_get_attrib_value( field_schema, "label", overrides) }" '
		attrib_str += f'message_err="{  JCFunc._fill_attribs_get_attrib_value( field_schema, "message_err", overrides) }" '
		attrib_str += f'type="{  JCFunc._fill_attribs_get_attrib_value( field_schema, "type", overrides) }" '
		attrib_str += f"validation='{  JCFunc._fill_attribs_get_attrib_value( field_schema, 'validation', overrides) }' " 	
		if data_value: attrib_str += f'value="{ data_value }" '

		return attrib_str

	######################################################################################################
	def _fill_attribs_get_attrib_value(   field_schema:dict, field_str:str, overrides:dict) -> str:
		field_value = ""
		if field_str in overrides:	#try to get from override first if present
			field_value = overrides[ field_str ]
		elif field_str == "validation": #if validation then use a json value-pair entry to validation field
			comma = ""
			for key in field_schema.get('validation',{} ).keys():
				comma += f'"{key}":"{ field_schema["validation"][key]}" ,'
			if comma: field_value += "{" + comma[:-1] + "}"
			# breakpoint()
		else:  #catch all - add the value from the database
			field_value = field_schema.get( field_str, "")
		return field_value

	######################################################################################################
	# 
	# <wc-input-text id="si_env_authsite_name"  class="ck_grp_elt" placeholder="Add site title"  label="Site Name"  validation='{"text_min_len":3}', message_err="Site name at least 3 characters" , value='defaultxx'   >
    #         </wc-input-text> 
	
	
					#   +
					#  f'validation='{"text_min_len":3}', message_err="Site name at least 3 characters" , value='defaultxx'   

	######################################################################################################
	# expand the url from the config files
	# scenario 1: url_for:member.home
	# scenario 2: url_for:members.home?var1=abc 
	# scenario 3: url_for:members.home?var1=abc&var2=xyz
	# scenario 3: url_for:members.home?var1=abc&var2=[data_field] abd data_table is a hash containing key 'data_field'
	def extract_url_from_config_str(urlStr, data_table = None):
		strTokens = re.split("[:\?]", urlStr )		#split into two parts, first bit into main link, then param
		arg_hash = {}

		if( len(strTokens) > 2):					#if we have parameters
			arg_list = re.split("[=&]", strTokens[2] )	#split each parameter

			#create dictionary.  arg_list[::2] = every seond item, arg_list[1::2]  = every first item
			arg_hash = dict( zip( arg_list[::2], arg_list[1::2] ) ) 

			if data_table:			#if we have data to replace
				for key in arg_hash:
					if arg_hash[key].startswith("[") and arg_hash[key].endswith("]"):	#if we have a bracked item
						arg_hash[key] = data_table.get(arg_hash[key][1:-1], arg_hash[key])	#check if key exists in table
	 
		return strTokens[1], arg_hash


	######################################################################################################
	# Custom Template Function: try to convert url_string
	def render_url(urlStr, data_table = None):

		if 'url_for' in urlStr:
			# strTokens = urlStr.split(":")
			# logger.debug("Tokens:" + str(strTokens))
			(link, args) = JCFunc.extract_url_from_config_str( urlStr, data_table )

			return url_for( link , **args)
		else:
			return  urlStr

	######################################################################################################
	# Custom Template Function: try to get url string, if not exist, return blank
	def url_for_lazy(urlStr ):

		try:
			return url_for( urlStr );
		except Exception as e:
			# logger.debug(f"URL converstaion failed:{urlStr}")
			return ""
		 
	 ######################################################################################################
	# enable creation of lists using the SET command under JINJA
	def make_list(*list_items): 
		return list_items

		
	



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mangosteen']

package_data = \
{'': ['*'], 'mangosteen': ['static/zcss/*', 'static/zjs/*', 'static/zwc/*']}

install_requires = \
['json-cfg', 'pymongo']

setup_kwargs = {
    'name': 'mangosteen',
    'version': '0.1.0',
    'description': 'Web widgets',
    'long_description': 'Web widgets for python',
    'author': 'pub12',
    'author_email': 'pubudu79@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import inspect, json

########################################################################################################################
########################################################################################################################
class MangosteenBase:
	def __init__(self, app=None, logger=None):
		self.logger = logger
		self.app = app
		self.debug = False

	def get_callee_str(self):
		callee = inspect.stack()[2]
		callee_file = callee.filename[ callee.filename.rfind('/'): ]

		ret_str = f"{callee_file }:{callee.lineno}:{callee.function}:" 
		# breakpoint()
		return ret_str

	def log_debug(self, message): 
		if self.logger: self.logger.debug( message, stack_level=2)
		else: print( "DEBUG:" + message )

	def log_error(self, message, stack_level=2):
		if self.logger: self.logger.error( message, stack_level)
		else: print( "ERROR:" + message )

	def log_info(self, message):
		if self.logger: self.logger.info( message, stack_level=2)
		else: print( "INFO:" + message )

	def log_warning(self, message):
		if self.logger: self.logger.warning( message, stack_level=2)
		else: print( "WARNING:" + message )

# ########################################################################################################################
# ########################################################################################################################
# class MangosteenBlueprint():
#     def __init__( self, blueprint_name, blueprint_module, route_class, url_prefix='/', template_folder='templates', static_folder='static', static_url_path='/static' ):
#         self._bp = Blueprint(   name=blueprint_name, 
#                                 import_name=blueprint_module, 
#                                 static_folder=static_folder,
#                                 static_url_path=static_url_path,
#                                 template_folder=template_folder,
#                                 url_prefix=url_prefix  )

#         self._route_class = route_class

#     def get_blueprint(self):
#         return self._bp

#     def get_view(self):
#         return self._route_class 
